@@ -8,24 +8,31 @@
  * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
  */
 
+#include <isl/id.h>
 #include <isl_space_private.h>
+#include <isl_val_private.h>
 #include <isl/set.h>
 #include <isl_reordering.h>
 
 #include <isl_multi_macro.h>
 
 #define MULTI_NAME(BASE) "isl_multi_" #BASE
-#define xLIST(EL) EL ## _list
-#define LIST(EL) xLIST(EL)
 
 isl_ctx *FN(MULTI(BASE),get_ctx)(__isl_keep MULTI(BASE) *multi)
 {
 	return multi ? isl_space_get_ctx(multi->space) : NULL;
 }
 
+/* Return the space of "multi".
+ */
+__isl_keep isl_space *FN(MULTI(BASE),peek_space)(__isl_keep MULTI(BASE) *multi)
+{
+	return multi ? multi->space : NULL;
+}
+
 __isl_give isl_space *FN(MULTI(BASE),get_space)(__isl_keep MULTI(BASE) *multi)
 {
-	return multi ? isl_space_copy(multi->space) : NULL;
+	return isl_space_copy(FN(MULTI(BASE),peek_space)(multi));
 }
 
 /* Return the position of the dimension of the given type and name
@@ -46,25 +53,36 @@ __isl_give isl_space *FN(MULTI(BASE),get_domain_space)(
 	return multi ? isl_space_domain(isl_space_copy(multi->space)) : NULL;
 }
 
+/* Allocate a multi expression living in "space".
+ *
+ * If the number of base expressions is zero, then make sure
+ * there is enough room in the structure for the explicit domain,
+ * in case the type supports such an explicit domain.
+ */
 __isl_give MULTI(BASE) *FN(MULTI(BASE),alloc)(__isl_take isl_space *space)
 {
 	isl_ctx *ctx;
-	int n;
+	isl_size n;
 	MULTI(BASE) *multi;
 
-	if (!space)
-		return NULL;
+	n = isl_space_dim(space, isl_dim_out);
+	if (n < 0)
+		goto error;
 
 	ctx = isl_space_get_ctx(space);
-	n = isl_space_dim(space, isl_dim_out);
-	multi = isl_calloc(ctx, MULTI(BASE),
+	if (n > 0)
+		multi = isl_calloc(ctx, MULTI(BASE),
 			 sizeof(MULTI(BASE)) + (n - 1) * sizeof(struct EL *));
+	else
+		multi = isl_calloc(ctx, MULTI(BASE), sizeof(MULTI(BASE)));
 	if (!multi)
 		goto error;
 
 	multi->space = space;
 	multi->n = n;
 	multi->ref = 1;
+	if (FN(MULTI(BASE),has_explicit_domain)(multi))
+		multi = FN(MULTI(BASE),init_explicit_domain)(multi);
 	return multi;
 error:
 	isl_space_free(space);
@@ -85,7 +103,9 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),dup)(__isl_keep MULTI(BASE) *multi)
 
 	for (i = 0; i < multi->n; ++i)
 		dup = FN(FN(MULTI(BASE),set),BASE)(dup, i,
-						    FN(EL,copy)(multi->p[i]));
+						    FN(EL,copy)(multi->u.p[i]));
+	if (FN(MULTI(BASE),has_explicit_domain)(multi))
+		dup = FN(MULTI(BASE),copy_explicit_domain)(dup, multi);
 
 	return dup;
 }
@@ -123,16 +143,18 @@ __isl_null MULTI(BASE) *FN(MULTI(BASE),free)(__isl_take MULTI(BASE) *multi)
 
 	isl_space_free(multi->space);
 	for (i = 0; i < multi->n; ++i)
-		FN(EL,free)(multi->p[i]);
+		FN(EL,free)(multi->u.p[i]);
+	if (FN(MULTI(BASE),has_explicit_domain)(multi))
+		FN(MULTI(BASE),free_explicit_domain)(multi);
 	free(multi);
 
 	return NULL;
 }
 
-unsigned FN(MULTI(BASE),dim)(__isl_keep MULTI(BASE) *multi,
+isl_size FN(MULTI(BASE),dim)(__isl_keep MULTI(BASE) *multi,
 	enum isl_dim_type type)
 {
-	return multi ? isl_space_dim(multi->space, type) : 0;
+	return isl_space_dim(FN(MULTI(BASE),peek_space)(multi), type);
 }
 
 /* Return the position of the first dimension of "type" with id "id".
@@ -171,8 +193,9 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),set_dim_name)(
 	if (type == isl_dim_out)
 		return multi;
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,set_dim_name)(multi->p[i], type, pos, s);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,set_dim_name)(multi->u.p[i],
+							type, pos, s);
+		if (!multi->u.p[i])
 			return FN(MULTI(BASE),free)(multi);
 	}
 
@@ -203,18 +226,59 @@ __isl_give isl_id *FN(MULTI(BASE),get_tuple_id)(__isl_keep MULTI(BASE) *multi,
 	return multi ? isl_space_get_tuple_id(multi->space, type) : NULL;
 }
 
+#undef TYPE
+#define TYPE	MULTI(BASE)
+static
+#include "check_type_range_templ.c"
+
 __isl_give EL *FN(FN(MULTI(BASE),get),BASE)(__isl_keep MULTI(BASE) *multi,
 	int pos)
 {
 	isl_ctx *ctx;
 
-	if (!multi)
+	if (FN(MULTI(BASE),check_range)(multi, isl_dim_out, pos, 1) < 0)
 		return NULL;
 	ctx = FN(MULTI(BASE),get_ctx)(multi);
-	if (pos < 0 || pos >= multi->n)
-		isl_die(ctx, isl_error_invalid,
-			"index out of bounds", return NULL);
-	return FN(EL,copy)(multi->p[pos]);
+	return FN(EL,copy)(multi->u.p[pos]);
+}
+
+/* Set the element at position "pos" of "multi" to "el",
+ * where the position may be empty if "multi" has only a single reference.
+ */
+static __isl_give MULTI(BASE) *FN(MULTI(BASE),restore)(
+	__isl_take MULTI(BASE) *multi, int pos, __isl_take EL *el)
+{
+	multi = FN(MULTI(BASE),cow)(multi);
+	if (!multi || !el)
+		goto error;
+
+	if (FN(MULTI(BASE),check_range)(multi, isl_dim_out, pos, 1) < 0)
+		goto error;
+
+	FN(EL,free)(multi->u.p[pos]);
+	multi->u.p[pos] = el;
+
+	return multi;
+error:
+	FN(MULTI(BASE),free)(multi);
+	FN(EL,free)(el);
+	return NULL;
+}
+
+/* Set the element at position "pos" of "multi" to "el",
+ * where the position may be empty if "multi" has only a single reference.
+ * However, the space of "multi" is available and is checked
+ * for compatibility with "el".
+ */
+static __isl_give MULTI(BASE) *FN(MULTI(BASE),restore_check_space)(
+	__isl_take MULTI(BASE) *multi, int pos, __isl_take EL *el)
+{
+	isl_space *space;
+
+	space = FN(MULTI(BASE),peek_space)(multi);
+	if (FN(EL,check_match_domain_space)(el, space) < 0)
+		multi = FN(MULTI(BASE),free)(multi);
+	return FN(MULTI(BASE),restore)(multi, pos, el);
 }
 
 __isl_give MULTI(BASE) *FN(FN(MULTI(BASE),set),BASE)(
@@ -224,10 +288,6 @@ __isl_give MULTI(BASE) *FN(FN(MULTI(BASE),set),BASE)(
 	isl_space *el_space = NULL;
 	isl_bool match;
 
-	multi = FN(MULTI(BASE),cow)(multi);
-	if (!multi || !el)
-		goto error;
-
 	multi_space = FN(MULTI(BASE),get_space)(multi);
 	match = FN(EL,matching_params)(el, multi_space);
 	if (match < 0)
@@ -239,15 +299,8 @@ __isl_give MULTI(BASE) *FN(FN(MULTI(BASE),set),BASE)(
 		multi_space = FN(MULTI(BASE),get_space)(multi);
 		el = FN(EL,align_params)(el, isl_space_copy(multi_space));
 	}
-	if (FN(EL,check_match_domain_space)(el, multi_space) < 0)
-		goto error;
 
-	if (pos < 0 || pos >= multi->n)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
-			"index out of bounds", goto error);
-
-	FN(EL,free)(multi->p[pos]);
-	multi->p[pos] = el;
+	multi = FN(MULTI(BASE),restore_check_space)(multi, pos, el);
 
 	isl_space_free(multi_space);
 	isl_space_free(el_space);
@@ -266,6 +319,10 @@ error:
  * directly or through its domain.  It therefore passes along both,
  * which we pass along to the element function since we don't know how
  * that is represented either.
+ *
+ * If "multi" has an explicit domain, then the caller is expected
+ * to make sure that any modification that would change the dimensions
+ * of the explicit domain has bee applied before this function is called.
  */
 __isl_give MULTI(BASE) *FN(MULTI(BASE),reset_space_and_domain)(
 	__isl_take MULTI(BASE) *multi, __isl_take isl_space *space,
@@ -278,9 +335,15 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),reset_space_and_domain)(
 		goto error;
 
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,reset_domain_space)(multi->p[i],
+		multi->u.p[i] = FN(EL,reset_domain_space)(multi->u.p[i],
 				 isl_space_copy(domain));
-		if (!multi->p[i])
+		if (!multi->u.p[i])
+			goto error;
+	}
+	if (FN(MULTI(BASE),has_explicit_domain)(multi)) {
+		multi = FN(MULTI(BASE),reset_explicit_domain_space)(multi,
+							isl_space_copy(domain));
+		if (!multi)
 			goto error;
 	}
 	isl_space_free(domain);
@@ -411,20 +474,21 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),realign_domain)(
 	__isl_take MULTI(BASE) *multi, __isl_take isl_reordering *exp)
 {
 	int i;
+	isl_space *space;
 
 	multi = FN(MULTI(BASE),cow)(multi);
 	if (!multi || !exp)
 		goto error;
 
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,realign_domain)(multi->p[i],
+		multi->u.p[i] = FN(EL,realign_domain)(multi->u.p[i],
 						isl_reordering_copy(exp));
-		if (!multi->p[i])
+		if (!multi->u.p[i])
 			goto error;
 	}
 
-	multi = FN(MULTI(BASE),reset_domain_space)(multi,
-						    isl_space_copy(exp->dim));
+	space = isl_reordering_get_space(exp);
+	multi = FN(MULTI(BASE),reset_domain_space)(multi, space);
 
 	isl_reordering_free(exp);
 	return multi;
@@ -435,6 +499,9 @@ error:
 }
 
 /* Align the parameters of "multi" to those of "model".
+ *
+ * If "multi" has an explicit domain, then align the parameters
+ * of the domain first.
  */
 __isl_give MULTI(BASE) *FN(MULTI(BASE),align_params)(
 	__isl_take MULTI(BASE) *multi, __isl_take isl_space *model)
@@ -462,7 +529,12 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),align_params)(
 		isl_die(ctx, isl_error_invalid,
 			"input has unnamed parameters", goto error);
 
-	model = isl_space_params(model);
+	if (FN(MULTI(BASE),has_explicit_domain)(multi)) {
+		multi = FN(MULTI(BASE),align_explicit_domain_params)(multi,
+							isl_space_copy(model));
+		if (!multi)
+			goto error;
+	}
 	exp = isl_parameter_alignment_reordering(multi->space, model);
 	exp = isl_reordering_extend_space(exp,
 				    FN(MULTI(BASE),get_domain_space)(multi));
@@ -476,27 +548,40 @@ error:
 	return NULL;
 }
 
+/* Create a multi expression in the given space with the elements of "list"
+ * as base expressions.
+ *
+ * Since isl_multi_*_restore_* assumes that the element and
+ * the multi expression have matching spaces, the alignment
+ * (if any) needs to be performed beforehand.
+ */
 __isl_give MULTI(BASE) *FN(FN(MULTI(BASE),from),LIST(BASE))(
 	__isl_take isl_space *space, __isl_take LIST(EL) *list)
 {
 	int i;
-	int n;
+	isl_size n, dim;
 	isl_ctx *ctx;
 	MULTI(BASE) *multi;
 
-	if (!space || !list)
+	dim = isl_space_dim(space, isl_dim_out);
+	n = FN(FN(LIST(EL),n),BASE)(list);
+	if (dim < 0 || n < 0)
 		goto error;
 
 	ctx = isl_space_get_ctx(space);
-	n = FN(FN(LIST(EL),n),BASE)(list);
-	if (n != isl_space_dim(space, isl_dim_out))
+	if (n != dim)
 		isl_die(ctx, isl_error_invalid,
 			"invalid number of elements in list", goto error);
 
+	for (i = 0; i < n; ++i) {
+		EL *el = FN(LIST(EL),peek)(list, i);
+		space = isl_space_align_params(space, FN(EL,get_space)(el));
+	}
 	multi = FN(MULTI(BASE),alloc)(isl_space_copy(space));
 	for (i = 0; i < n; ++i) {
-		multi = FN(FN(MULTI(BASE),set),BASE)(multi, i,
-					FN(FN(LIST(EL),get),BASE)(list, i));
+		EL *el = FN(FN(LIST(EL),get),BASE)(list, i);
+		el = FN(EL,align_params)(el, isl_space_copy(space));
+		multi = FN(MULTI(BASE),restore_check_space)(multi, i, el);
 	}
 
 	isl_space_free(space);
@@ -508,135 +593,15 @@ error:
 	return NULL;
 }
 
-#ifndef NO_IDENTITY
-/* Create a multi expression in the given space that maps each
- * input dimension to the corresponding output dimension.
- */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),identity)(__isl_take isl_space *space)
-{
-	int i, n;
-	isl_local_space *ls;
-	MULTI(BASE) *multi;
-
-	if (!space)
-		return NULL;
-
-	if (isl_space_is_set(space))
-		isl_die(isl_space_get_ctx(space), isl_error_invalid,
-			"expecting map space", goto error);
-
-	n = isl_space_dim(space, isl_dim_out);
-	if (n != isl_space_dim(space, isl_dim_in))
-		isl_die(isl_space_get_ctx(space), isl_error_invalid,
-			"number of input and output dimensions needs to be "
-			"the same", goto error);
-
-	multi = FN(MULTI(BASE),alloc)(isl_space_copy(space));
-
-	if (!n) {
-		isl_space_free(space);
-		return multi;
-	}
-
-	space = isl_space_domain(space);
-	ls = isl_local_space_from_space(space);
-
-	for (i = 0; i < n; ++i) {
-		EL *el;
-		el = FN(EL,var_on_domain)(isl_local_space_copy(ls),
-						isl_dim_set, i);
-		multi = FN(FN(MULTI(BASE),set),BASE)(multi, i, el);
-	}
-
-	isl_local_space_free(ls);
-
-	return multi;
-error:
-	isl_space_free(space);
-	return NULL;
-}
-#endif
-
-#ifndef NO_ZERO
-/* Construct a multi expression in the given space with value zero in
- * each of the output dimensions.
- */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),zero)(__isl_take isl_space *space)
-{
-	int n;
-	MULTI(BASE) *multi;
-
-	if (!space)
-		return NULL;
-
-	n = isl_space_dim(space , isl_dim_out);
-	multi = FN(MULTI(BASE),alloc)(isl_space_copy(space));
-
-	if (!n)
-		isl_space_free(space);
-	else {
-		int i;
-		isl_local_space *ls;
-		EL *el;
-
-		space = isl_space_domain(space);
-		ls = isl_local_space_from_space(space);
-		el = FN(EL,zero_on_domain)(ls);
-
-		for (i = 0; i < n; ++i)
-			multi = FN(FN(MULTI(BASE),set),BASE)(multi, i,
-							    FN(EL,copy)(el));
-
-		FN(EL,free)(el);
-	}
-
-	return multi;
-}
-#endif
-
-#ifndef NO_FROM_BASE
-/* Create a multiple expression with a single output/set dimension
- * equal to "el".
- * For most multiple expression types, the base type has a single
- * output/set dimension and the space of the result is therefore
- * the same as the space of the input.
- * In the case of isl_multi_union_pw_aff, however, the base type
- * lives in a parameter space and we therefore need to add
- * a single set dimension.
- */
-__isl_give MULTI(BASE) *FN(FN(MULTI(BASE),from),BASE)(__isl_take EL *el)
-{
-	isl_space *space;
-	MULTI(BASE) *multi;
-
-	space = FN(EL,get_space(el));
-	if (isl_space_is_params(space)) {
-		space = isl_space_set_from_params(space);
-		space = isl_space_add_dims(space, isl_dim_set, 1);
-	}
-	multi = FN(MULTI(BASE),alloc)(space);
-	multi = FN(FN(MULTI(BASE),set),BASE)(multi, 0, el);
-
-	return multi;
-}
-#endif
-
 __isl_give MULTI(BASE) *FN(MULTI(BASE),drop_dims)(
 	__isl_take MULTI(BASE) *multi,
 	enum isl_dim_type type, unsigned first, unsigned n)
 {
 	int i;
-	unsigned dim;
 
 	multi = FN(MULTI(BASE),cow)(multi);
-	if (!multi)
-		return NULL;
-
-	dim = FN(MULTI(BASE),dim)(multi, type);
-	if (first + n > dim || first + n < first)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
-			"index out of bounds",
-			return FN(MULTI(BASE),free)(multi));
+	if (FN(MULTI(BASE),check_range)(multi, type, first, n) < 0)
+		return FN(MULTI(BASE),free)(multi);
 
 	multi->space = isl_space_drop_dims(multi->space, type, first, n);
 	if (!multi->space)
@@ -644,17 +609,25 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),drop_dims)(
 
 	if (type == isl_dim_out) {
 		for (i = 0; i < n; ++i)
-			FN(EL,free)(multi->p[first + i]);
+			FN(EL,free)(multi->u.p[first + i]);
 		for (i = first; i + n < multi->n; ++i)
-			multi->p[i] = multi->p[i + n];
+			multi->u.p[i] = multi->u.p[i + n];
 		multi->n -= n;
+		if (n > 0 && FN(MULTI(BASE),has_explicit_domain)(multi))
+			multi = FN(MULTI(BASE),init_explicit_domain)(multi);
 
 		return multi;
 	}
 
+	if (FN(MULTI(BASE),has_explicit_domain)(multi))
+		multi = FN(MULTI(BASE),drop_explicit_domain_dims)(multi,
+								type, first, n);
+	if (!multi)
+		return NULL;
+
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,drop_dims)(multi->p[i], type, first, n);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,drop_dims)(multi->u.p[i], type, first, n);
+		if (!multi->u.p[i])
 			return FN(MULTI(BASE),free)(multi);
 	}
 
@@ -698,25 +671,28 @@ error:
  * construct a MULTI(BASE) (A * C) -> [B -> D].
  *
  * The parameters are assumed to have been aligned.
+ *
+ * If "multi1" and/or "multi2" has an explicit domain, then
+ * intersect the domain of the result with these explicit domains.
  */
 static __isl_give MULTI(BASE) *FN(MULTI(BASE),range_product_aligned)(
 	__isl_take MULTI(BASE) *multi1, __isl_take MULTI(BASE) *multi2)
 {
-	int i, n1, n2;
+	int i;
+	isl_size n1, n2;
 	EL *el;
 	isl_space *space;
 	MULTI(BASE) *res;
 
-	if (!multi1 || !multi2)
+	n1 = FN(MULTI(BASE),dim)(multi1, isl_dim_out);
+	n2 = FN(MULTI(BASE),dim)(multi2, isl_dim_out);
+	if (n1 < 0 || n2 < 0)
 		goto error;
 
 	space = isl_space_range_product(FN(MULTI(BASE),get_space)(multi1),
 					FN(MULTI(BASE),get_space)(multi2));
 	res = FN(MULTI(BASE),alloc)(space);
 
-	n1 = FN(MULTI(BASE),dim)(multi1, isl_dim_out);
-	n2 = FN(MULTI(BASE),dim)(multi2, isl_dim_out);
-
 	for (i = 0; i < n1; ++i) {
 		el = FN(FN(MULTI(BASE),get),BASE)(multi1, i);
 		res = FN(FN(MULTI(BASE),set),BASE)(res, i, el);
@@ -727,6 +703,11 @@ static __isl_give MULTI(BASE) *FN(MULTI(BASE),range_product_aligned)(
 		res = FN(FN(MULTI(BASE),set),BASE)(res, n1 + i, el);
 	}
 
+	if (FN(MULTI(BASE),has_explicit_domain)(multi1))
+		res = FN(MULTI(BASE),intersect_explicit_domain)(res, multi1);
+	if (FN(MULTI(BASE),has_explicit_domain)(multi2))
+		res = FN(MULTI(BASE),intersect_explicit_domain)(res, multi2);
+
 	FN(MULTI(BASE),free)(multi1);
 	FN(MULTI(BASE),free)(multi2);
 	return res;
@@ -761,19 +742,21 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),range_factor_domain)(
 	__isl_take MULTI(BASE) *multi)
 {
 	isl_space *space;
-	int total, keep;
+	isl_size total, keep;
 
-	if (!multi)
-		return NULL;
+	total = FN(MULTI(BASE),dim)(multi, isl_dim_out);
+	if (total < 0)
+		return FN(MULTI(BASE),free)(multi);
 	if (!isl_space_range_is_wrapping(multi->space))
 		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
 			"range is not a product",
 			return FN(MULTI(BASE),free)(multi));
 
 	space = FN(MULTI(BASE),get_space)(multi);
-	total = isl_space_dim(space, isl_dim_out);
 	space = isl_space_range_factor_domain(space);
 	keep = isl_space_dim(space, isl_dim_out);
+	if (keep < 0)
+		multi = FN(MULTI(BASE),free)(multi);
 	multi = FN(MULTI(BASE),drop_dims)(multi,
 					isl_dim_out, keep, total - keep);
 	multi = FN(MULTI(BASE),reset_space)(multi, space);
@@ -787,19 +770,21 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),range_factor_range)(
 	__isl_take MULTI(BASE) *multi)
 {
 	isl_space *space;
-	int total, keep;
+	isl_size total, keep;
 
-	if (!multi)
-		return NULL;
+	total = FN(MULTI(BASE),dim)(multi, isl_dim_out);
+	if (total < 0)
+		return FN(MULTI(BASE),free)(multi);
 	if (!isl_space_range_is_wrapping(multi->space))
 		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
 			"range is not a product",
 			return FN(MULTI(BASE),free)(multi));
 
 	space = FN(MULTI(BASE),get_space)(multi);
-	total = isl_space_dim(space, isl_dim_out);
 	space = isl_space_range_factor_range(space);
 	keep = isl_space_dim(space, isl_dim_out);
+	if (keep < 0)
+		multi = FN(MULTI(BASE),free)(multi);
 	multi = FN(MULTI(BASE),drop_dims)(multi, isl_dim_out, 0, total - keep);
 	multi = FN(MULTI(BASE),reset_space)(multi, space);
 
@@ -812,79 +797,26 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),factor_range)(
 	__isl_take MULTI(BASE) *multi)
 {
 	isl_space *space;
-	int total, keep;
+	isl_size total, keep;
 
-	if (!multi)
-		return NULL;
+	total = FN(MULTI(BASE),dim)(multi, isl_dim_set);
+	if (total < 0)
+		return FN(MULTI(BASE),free)(multi);
 	if (!isl_space_is_wrapping(multi->space))
 		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
 			"not a product", return FN(MULTI(BASE),free)(multi));
 
 	space = FN(MULTI(BASE),get_space)(multi);
-	total = isl_space_dim(space, isl_dim_out);
 	space = isl_space_factor_range(space);
-	keep = isl_space_dim(space, isl_dim_out);
-	multi = FN(MULTI(BASE),drop_dims)(multi, isl_dim_out, 0, total - keep);
+	keep = isl_space_dim(space, isl_dim_set);
+	if (keep < 0)
+		multi = FN(MULTI(BASE),free)(multi);
+	multi = FN(MULTI(BASE),drop_dims)(multi, isl_dim_set, 0, total - keep);
 	multi = FN(MULTI(BASE),reset_space)(multi, space);
 
 	return multi;
 }
 
-#ifndef NO_PRODUCT
-/* Given two MULTI(BASE)s A -> B and C -> D,
- * construct a MULTI(BASE) [A -> C] -> [B -> D].
- *
- * The parameters are assumed to have been aligned.
- */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),product_aligned)(
-	__isl_take MULTI(BASE) *multi1, __isl_take MULTI(BASE) *multi2)
-{
-	int i;
-	EL *el;
-	isl_space *space;
-	MULTI(BASE) *res;
-	int in1, in2, out1, out2;
-
-	in1 = FN(MULTI(BASE),dim)(multi1, isl_dim_in);
-	in2 = FN(MULTI(BASE),dim)(multi2, isl_dim_in);
-	out1 = FN(MULTI(BASE),dim)(multi1, isl_dim_out);
-	out2 = FN(MULTI(BASE),dim)(multi2, isl_dim_out);
-	space = isl_space_product(FN(MULTI(BASE),get_space)(multi1),
-				  FN(MULTI(BASE),get_space)(multi2));
-	res = FN(MULTI(BASE),alloc)(isl_space_copy(space));
-	space = isl_space_domain(space);
-
-	for (i = 0; i < out1; ++i) {
-		el = FN(FN(MULTI(BASE),get),BASE)(multi1, i);
-		el = FN(EL,insert_dims)(el, isl_dim_in, in1, in2);
-		el = FN(EL,reset_domain_space)(el, isl_space_copy(space));
-		res = FN(FN(MULTI(BASE),set),BASE)(res, i, el);
-	}
-
-	for (i = 0; i < out2; ++i) {
-		el = FN(FN(MULTI(BASE),get),BASE)(multi2, i);
-		el = FN(EL,insert_dims)(el, isl_dim_in, 0, in1);
-		el = FN(EL,reset_domain_space)(el, isl_space_copy(space));
-		res = FN(FN(MULTI(BASE),set),BASE)(res, out1 + i, el);
-	}
-
-	isl_space_free(space);
-	FN(MULTI(BASE),free)(multi1);
-	FN(MULTI(BASE),free)(multi2);
-	return res;
-}
-
-/* Given two MULTI(BASE)s A -> B and C -> D,
- * construct a MULTI(BASE) [A -> C] -> [B -> D].
- */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),product)(
-	__isl_take MULTI(BASE) *multi1, __isl_take MULTI(BASE) *multi2)
-{
-	return FN(MULTI(BASE),align_params_multi_multi_and)(multi1, multi2,
-					&FN(MULTI(BASE),product_aligned));
-}
-#endif
-
 __isl_give MULTI(BASE) *FN(MULTI(BASE),flatten_range)(
 	__isl_take MULTI(BASE) *multi)
 {
@@ -935,15 +867,14 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),range_splice)(
 	__isl_take MULTI(BASE) *multi2)
 {
 	MULTI(BASE) *res;
-	unsigned dim;
-
-	if (!multi1 || !multi2)
-		goto error;
+	isl_size dim;
 
 	dim = FN(MULTI(BASE),dim)(multi1, isl_dim_out);
-	if (pos > dim)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi1), isl_error_invalid,
-			"index out of bounds", goto error);
+	if (dim < 0 || !multi2)
+		goto error;
+
+	if (FN(MULTI(BASE),check_range)(multi1, isl_dim_out, pos, 0) < 0)
+		goto error;
 
 	res = FN(MULTI(BASE),copy)(multi1);
 	res = FN(MULTI(BASE),drop_dims)(res, isl_dim_out, pos, dim - pos);
@@ -959,59 +890,26 @@ error:
 	return NULL;
 }
 
-#ifndef NO_SPLICE
-/* Given two multi expressions, "multi1"
- *
- *	[A1 A2] -> [B1 B2]
- *
- * where A2 starts at position "in_pos" and B2 starts at position "out_pos",
- * and "multi2"
- *
- *	[C] -> [D]
- *
- * return the multi expression
- *
- *	[A1 C A2] -> [B1 D B2]
- *
- * We first insert input dimensions to obtain
- *
- *	[A1 C A2] -> [B1 B2]
- *
- * and
- *
- *	[A1 C A2] -> [D]
- *
- * and then apply range_splice.
+/* Check that "multi1" and "multi2" live in the same space,
+ * reporting an error if they do not.
  */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),splice)(
-	__isl_take MULTI(BASE) *multi1, unsigned in_pos, unsigned out_pos,
-	__isl_take MULTI(BASE) *multi2)
+static isl_stat FN(MULTI(BASE),check_equal_space)(
+	__isl_keep MULTI(BASE) *multi1, __isl_keep MULTI(BASE) *multi2)
 {
-	unsigned n_in1;
-	unsigned n_in2;
+	isl_bool equal;
 
 	if (!multi1 || !multi2)
-		goto error;
+		return isl_stat_error;
 
-	n_in1 = FN(MULTI(BASE),dim)(multi1, isl_dim_in);
-	if (in_pos > n_in1)
+	equal = isl_space_is_equal(multi1->space, multi2->space);
+	if (equal < 0)
+		return isl_stat_error;
+	if (!equal)
 		isl_die(FN(MULTI(BASE),get_ctx)(multi1), isl_error_invalid,
-			"index out of bounds", goto error);
+			"spaces don't match", return isl_stat_error);
 
-	n_in2 = FN(MULTI(BASE),dim)(multi2, isl_dim_in);
-
-	multi1 = FN(MULTI(BASE),insert_dims)(multi1, isl_dim_in, in_pos, n_in2);
-	multi2 = FN(MULTI(BASE),insert_dims)(multi2, isl_dim_in, n_in2,
-						n_in1 - in_pos);
-	multi2 = FN(MULTI(BASE),insert_dims)(multi2, isl_dim_in, 0, in_pos);
-
-	return FN(MULTI(BASE),range_splice)(multi1, out_pos, multi2);
-error:
-	FN(MULTI(BASE),free)(multi1);
-	FN(MULTI(BASE),free)(multi2);
-	return NULL;
+	return isl_stat_ok;
 }
-#endif
 
 /* This function is currently only used from isl_aff.c
  */
@@ -1022,29 +920,31 @@ static __isl_give MULTI(BASE) *FN(MULTI(BASE),bin_op)(
 
 /* Pairwise perform "fn" to the elements of "multi1" and "multi2" and
  * return the result.
+ *
+ * If "multi2" has an explicit domain, then
+ * intersect the domain of the result with this explicit domain.
  */
 static __isl_give MULTI(BASE) *FN(MULTI(BASE),bin_op)(
 	__isl_take MULTI(BASE) *multi1, __isl_take MULTI(BASE) *multi2,
 	__isl_give EL *(*fn)(__isl_take EL *, __isl_take EL *))
 {
 	int i;
-	isl_ctx *ctx;
 
 	multi1 = FN(MULTI(BASE),cow)(multi1);
-	if (!multi1 || !multi2)
+	if (FN(MULTI(BASE),check_equal_space)(multi1, multi2) < 0)
 		goto error;
 
-	ctx = FN(MULTI(BASE),get_ctx)(multi1);
-	if (!isl_space_is_equal(multi1->space, multi2->space))
-		isl_die(ctx, isl_error_invalid,
-			"spaces don't match", goto error);
-
 	for (i = 0; i < multi1->n; ++i) {
-		multi1->p[i] = fn(multi1->p[i], FN(EL,copy)(multi2->p[i]));
-		if (!multi1->p[i])
+		multi1->u.p[i] = fn(multi1->u.p[i],
+						FN(EL,copy)(multi2->u.p[i]));
+		if (!multi1->u.p[i])
 			goto error;
 	}
 
+	if (FN(MULTI(BASE),has_explicit_domain)(multi2))
+		multi1 = FN(MULTI(BASE),intersect_explicit_domain)(multi1,
+								    multi2);
+
 	FN(MULTI(BASE),free)(multi2);
 	return multi1;
 error:
@@ -1053,7 +953,7 @@ error:
 	return NULL;
 }
 
-/* Add "multi2" from "multi1" and return the result.
+/* Add "multi2" to "multi1" and return the result.
  *
  * The parameters of "multi1" and "multi2" are assumed to have been aligned.
  */
@@ -1063,7 +963,7 @@ static __isl_give MULTI(BASE) *FN(MULTI(BASE),add_aligned)(
 	return FN(MULTI(BASE),bin_op)(multi1, multi2, &FN(EL,add));
 }
 
-/* Add "multi2" from "multi1" and return the result.
+/* Add "multi2" to "multi1" and return the result.
  */
 __isl_give MULTI(BASE) *FN(MULTI(BASE),add)(__isl_take MULTI(BASE) *multi1,
 	__isl_take MULTI(BASE) *multi2)
@@ -1115,8 +1015,9 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),scale_val)(__isl_take MULTI(BASE) *multi,
 		return NULL;
 
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,scale_val)(multi->p[i], isl_val_copy(v));
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,scale_val)(multi->u.p[i],
+						isl_val_copy(v));
+		if (!multi->u.p[i])
 			goto error;
 	}
 
@@ -1154,9 +1055,9 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),scale_down_val)(
 		return NULL;
 
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,scale_down_val)(multi->p[i],
+		multi->u.p[i] = FN(EL,scale_down_val)(multi->u.p[i],
 						    isl_val_copy(v));
-		if (!multi->p[i])
+		if (!multi->u.p[i])
 			goto error;
 	}
 
@@ -1191,8 +1092,8 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),scale_multi_val)(
 		isl_val *v;
 
 		v = isl_multi_val_get_val(mv, i);
-		multi->p[i] = FN(EL,scale_val)(multi->p[i], v);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,scale_val)(multi->u.p[i], v);
+		if (!multi->u.p[i])
 			goto error;
 	}
 
@@ -1227,8 +1128,8 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),scale_down_multi_val)(
 		isl_val *v;
 
 		v = isl_multi_val_get_val(mv, i);
-		multi->p[i] = FN(EL,scale_down_val)(multi->p[i], v);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,scale_down_val)(multi->u.p[i], v);
+		if (!multi->u.p[i])
 			goto error;
 	}
 
@@ -1263,8 +1164,8 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),mod_multi_val)(
 		isl_val *v;
 
 		v = isl_multi_val_get_val(mv, i);
-		multi->p[i] = FN(EL,mod_val)(multi->p[i], v);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,mod_val)(multi->u.p[i], v);
+		if (!multi->u.p[i])
 			goto error;
 	}
 
@@ -1275,63 +1176,6 @@ error:
 	return FN(MULTI(BASE),free)(multi);
 }
 
-#ifndef NO_MOVE_DIMS
-/* Move the "n" dimensions of "src_type" starting at "src_pos" of "multi"
- * to dimensions of "dst_type" at "dst_pos".
- *
- * We only support moving input dimensions to parameters and vice versa.
- */
-__isl_give MULTI(BASE) *FN(MULTI(BASE),move_dims)(__isl_take MULTI(BASE) *multi,
-	enum isl_dim_type dst_type, unsigned dst_pos,
-	enum isl_dim_type src_type, unsigned src_pos, unsigned n)
-{
-	int i;
-
-	if (!multi)
-		return NULL;
-
-	if (n == 0 &&
-	    !isl_space_is_named_or_nested(multi->space, src_type) &&
-	    !isl_space_is_named_or_nested(multi->space, dst_type))
-		return multi;
-
-	if (dst_type == isl_dim_out || src_type == isl_dim_out)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
-			"cannot move output/set dimension",
-			return FN(MULTI(BASE),free)(multi));
-	if (dst_type == isl_dim_div || src_type == isl_dim_div)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
-			"cannot move divs",
-			return FN(MULTI(BASE),free)(multi));
-	if (src_pos + n > isl_space_dim(multi->space, src_type))
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_invalid,
-			"range out of bounds",
-			return FN(MULTI(BASE),free)(multi));
-	if (dst_type == src_type)
-		isl_die(FN(MULTI(BASE),get_ctx)(multi), isl_error_unsupported,
-			"moving dims within the same type not supported",
-			return FN(MULTI(BASE),free)(multi));
-
-	multi = FN(MULTI(BASE),cow)(multi);
-	if (!multi)
-		return NULL;
-
-	multi->space = isl_space_move_dims(multi->space, dst_type, dst_pos,
-						src_type, src_pos, n);
-	if (!multi->space)
-		return FN(MULTI(BASE),free)(multi);
-
-	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,move_dims)(multi->p[i], dst_type, dst_pos,
-						src_type, src_pos, n);
-		if (!multi->p[i])
-			return FN(MULTI(BASE),free)(multi);
-	}
-
-	return multi;
-}
-#endif
-
 /* Convert a multiple expression defined over a parameter domain
  * into one that is defined over a zero-dimensional set.
  */
@@ -1371,7 +1215,14 @@ isl_bool FN(MULTI(BASE),plain_is_equal)(__isl_keep MULTI(BASE) *multi1,
 		return equal;
 
 	for (i = 0; i < multi1->n; ++i) {
-		equal = FN(EL,plain_is_equal)(multi1->p[i], multi2->p[i]);
+		equal = FN(EL,plain_is_equal)(multi1->u.p[i], multi2->u.p[i]);
+		if (equal < 0 || !equal)
+			return equal;
+	}
+
+	if (FN(MULTI(BASE),has_explicit_domain)(multi1) ||
+	    FN(MULTI(BASE),has_explicit_domain)(multi2)) {
+		equal = FN(MULTI(BASE),equal_explicit_domain)(multi1, multi2);
 		if (equal < 0 || !equal)
 			return equal;
 	}
@@ -1391,7 +1242,7 @@ isl_bool FN(MULTI(BASE),involves_nan)(__isl_keep MULTI(BASE) *multi)
 		return isl_bool_false;
 
 	for (i = 0; i < multi->n; ++i) {
-		isl_bool has_nan = FN(EL,involves_nan)(multi->p[i]);
+		isl_bool has_nan = FN(EL,involves_nan)(multi->u.p[i]);
 		if (has_nan < 0 || has_nan)
 			return has_nan;
 	}
@@ -1399,31 +1250,6 @@ isl_bool FN(MULTI(BASE),involves_nan)(__isl_keep MULTI(BASE) *multi)
 	return isl_bool_false;
 }
 
-#ifndef NO_DOMAIN
-/* Return the shared domain of the elements of "multi".
- */
-__isl_give isl_set *FN(MULTI(BASE),domain)(__isl_take MULTI(BASE) *multi)
-{
-	int i;
-	isl_set *dom;
-
-	if (!multi)
-		return NULL;
-
-	dom = isl_set_universe(FN(MULTI(BASE),get_domain_space)(multi));
-	for (i = 0; i < multi->n; ++i) {
-		isl_set *dom_i;
-
-		dom_i = FN(EL,domain)(FN(FN(MULTI(BASE),get),BASE)(multi, i));
-		dom = isl_set_intersect(dom, dom_i);
-	}
-
-	FN(MULTI(BASE),free)(multi);
-	return dom;
-}
-#endif
-
-#ifndef NO_NEG
 /* Return the opposite of "multi".
  */
 __isl_give MULTI(BASE) *FN(MULTI(BASE),neg)(__isl_take MULTI(BASE) *multi)
@@ -1435,11 +1261,10 @@ __isl_give MULTI(BASE) *FN(MULTI(BASE),neg)(__isl_take MULTI(BASE) *multi)
 		return NULL;
 
 	for (i = 0; i < multi->n; ++i) {
-		multi->p[i] = FN(EL,neg)(multi->p[i]);
-		if (!multi->p[i])
+		multi->u.p[i] = FN(EL,neg)(multi->u.p[i]);
+		if (!multi->u.p[i])
 			return FN(MULTI(BASE),free)(multi);
 	}
 
 	return multi;
 }
-#endif
