@@ -9,6 +9,7 @@
  */
 
 #include <isl_ctx_private.h>
+#include <isl/id.h>
 #include <isl_space_private.h>
 #include <isl_reordering.h>
 
@@ -23,7 +24,7 @@ __isl_give isl_reordering *isl_reordering_alloc(isl_ctx *ctx, int len)
 
 	exp->ref = 1;
 	exp->len = len;
-	exp->dim = NULL;
+	exp->space = NULL;
 
 	return exp;
 }
@@ -45,12 +46,12 @@ __isl_give isl_reordering *isl_reordering_dup(__isl_keep isl_reordering *r)
 	if (!r)
 		return NULL;
 
-	dup = isl_reordering_alloc(r->dim->ctx, r->len);
+	dup = isl_reordering_alloc(isl_reordering_get_ctx(r), r->len);
 	if (!dup)
 		return NULL;
 
-	dup->dim = isl_space_copy(r->dim);
-	if (!dup->dim)
+	dup->space = isl_reordering_get_space(r);
+	if (!dup->space)
 		return isl_reordering_free(dup);
 	for (i = 0; i < dup->len; ++i)
 		dup->pos[i] = r->pos[i];
@@ -69,7 +70,7 @@ __isl_give isl_reordering *isl_reordering_cow(__isl_take isl_reordering *r)
 	return isl_reordering_dup(r);
 }
 
-void *isl_reordering_free(__isl_take isl_reordering *exp)
+__isl_null isl_reordering *isl_reordering_free(__isl_take isl_reordering *exp)
 {
 	if (!exp)
 		return NULL;
@@ -77,11 +78,34 @@ void *isl_reordering_free(__isl_take isl_reordering *exp)
 	if (--exp->ref > 0)
 		return NULL;
 
-	isl_space_free(exp->dim);
+	isl_space_free(exp->space);
 	free(exp);
 	return NULL;
 }
 
+/* Return the isl_ctx to which "r" belongs.
+ */
+isl_ctx *isl_reordering_get_ctx(__isl_keep isl_reordering *r)
+{
+	return isl_space_get_ctx(isl_reordering_peek_space(r));
+}
+
+/* Return the space of "r".
+ */
+__isl_keep isl_space *isl_reordering_peek_space(__isl_keep isl_reordering *r)
+{
+	if (!r)
+		return NULL;
+	return r->space;
+}
+
+/* Return a copy of the space of "r".
+ */
+__isl_give isl_space *isl_reordering_get_space(__isl_keep isl_reordering *r)
+{
+	return isl_space_copy(isl_reordering_peek_space(r));
+}
+
 /* Construct a reordering that maps the parameters of "alignee"
  * to the corresponding parameters in a new dimension specification
  * that has the parameters of "aligner" first, followed by
@@ -100,7 +124,7 @@ __isl_give isl_reordering *isl_parameter_alignment_reordering(
 	if (!exp)
 		return NULL;
 
-	exp->dim = isl_space_copy(aligner);
+	exp->space = isl_space_params(isl_space_copy(aligner));
 
 	for (i = 0; i < alignee->nparam; ++i) {
 		isl_id *id_i;
@@ -119,16 +143,19 @@ __isl_give isl_reordering *isl_parameter_alignment_reordering(
 			exp->pos[i] = j;
 			isl_id_free(id_i);
 		} else {
-			int pos;
-			pos = isl_space_dim(exp->dim, isl_dim_param);
-			exp->dim = isl_space_add_dims(exp->dim, isl_dim_param, 1);
-			exp->dim = isl_space_set_dim_id(exp->dim,
+			isl_size pos;
+			pos = isl_space_dim(exp->space, isl_dim_param);
+			if (pos < 0)
+				exp->space = isl_space_free(exp->space);
+			exp->space = isl_space_add_dims(exp->space,
+						isl_dim_param, 1);
+			exp->space = isl_space_set_dim_id(exp->space,
 						isl_dim_param, pos, id_i);
 			exp->pos[i] = pos;
 		}
 	}
 
-	if (!exp->dim)
+	if (!exp->space)
 		return isl_reordering_free(exp);
 	return exp;
 error:
@@ -140,19 +167,27 @@ __isl_give isl_reordering *isl_reordering_extend(__isl_take isl_reordering *exp,
 	unsigned extra)
 {
 	int i;
+	isl_ctx *ctx;
+	isl_space *space;
 	isl_reordering *res;
 	int offset;
+	isl_size dim;
 
 	if (!exp)
 		return NULL;
 	if (extra == 0)
 		return exp;
 
-	offset = isl_space_dim(exp->dim, isl_dim_all) - exp->len;
-	res = isl_reordering_alloc(exp->dim->ctx, exp->len + extra);
+	ctx = isl_reordering_get_ctx(exp);
+	space = isl_reordering_peek_space(exp);
+	dim = isl_space_dim(space, isl_dim_all);
+	if (dim < 0)
+		return isl_reordering_free(exp);
+	offset = dim - exp->len;
+	res = isl_reordering_alloc(ctx, exp->len + extra);
 	if (!res)
 		goto error;
-	res->dim = isl_space_copy(exp->dim);
+	res->space = isl_reordering_get_space(exp);
 	for (i = 0; i < exp->len; ++i)
 		res->pos[i] = exp->pos[i];
 	for (i = exp->len; i < res->len; ++i)
@@ -169,22 +204,25 @@ error:
 __isl_give isl_reordering *isl_reordering_extend_space(
 	__isl_take isl_reordering *exp, __isl_take isl_space *space)
 {
+	isl_space *exp_space;
 	isl_reordering *res;
+	isl_size dim;
 
-	if (!exp || !space)
+	dim = isl_space_dim(space, isl_dim_all);
+	if (!exp || dim < 0)
 		goto error;
 
-	res = isl_reordering_extend(isl_reordering_copy(exp),
-				isl_space_dim(space, isl_dim_all) - exp->len);
+	res = isl_reordering_extend(isl_reordering_copy(exp), dim - exp->len);
 	res = isl_reordering_cow(res);
 	if (!res)
 		goto error;
-	isl_space_free(res->dim);
-	res->dim = isl_space_replace_params(space, exp->dim);
+	isl_space_free(res->space);
+	exp_space = isl_reordering_peek_space(exp);
+	res->space = isl_space_replace_params(space, exp_space);
 
 	isl_reordering_free(exp);
 
-	if (!res->dim)
+	if (!res->space)
 		return isl_reordering_free(res);
 
 	return res;
@@ -198,7 +236,7 @@ void isl_reordering_dump(__isl_keep isl_reordering *exp)
 {
 	int i;
 
-	isl_space_dump(exp->dim);
+	isl_space_dump(exp->space);
 	for (i = 0; i < exp->len; ++i)
 		fprintf(stderr, "%d -> %d; ", i, exp->pos[i]);
 	fprintf(stderr, "\n");
