@@ -48,19 +48,21 @@ static __isl_give isl_schedule_band *isl_schedule_band_alloc(isl_ctx *ctx)
 __isl_give isl_schedule_band *isl_schedule_band_from_multi_union_pw_aff(
 	__isl_take isl_multi_union_pw_aff *mupa)
 {
+	isl_size dim;
 	isl_ctx *ctx;
 	isl_schedule_band *band;
 	isl_space *space;
 
 	mupa = isl_multi_union_pw_aff_floor(mupa);
-	if (!mupa)
-		return NULL;
+	dim = isl_multi_union_pw_aff_dim(mupa, isl_dim_set);
+	if (dim < 0)
+		goto error;
 	ctx = isl_multi_union_pw_aff_get_ctx(mupa);
 	band = isl_schedule_band_alloc(ctx);
 	if (!band)
 		goto error;
 
-	band->n = isl_multi_union_pw_aff_dim(mupa, isl_dim_set);
+	band->n = dim;
 	band->coincident = isl_calloc_array(ctx, int, band->n);
 	band->mupa = mupa;
 	space = isl_space_params_alloc(ctx, 0);
@@ -221,9 +223,9 @@ isl_bool isl_schedule_band_plain_is_equal(__isl_keep isl_schedule_band *band1,
 
 /* Return the number of scheduling dimensions in the band.
  */
-int isl_schedule_band_n_member(__isl_keep isl_schedule_band *band)
+isl_size isl_schedule_band_n_member(__isl_keep isl_schedule_band *band)
 {
-	return band ? band->n : 0;
+	return band ? band->n : isl_size_error;
 }
 
 /* Is the given scheduling dimension coincident within the band and
@@ -239,7 +241,7 @@ isl_bool isl_schedule_band_member_get_coincident(
 		isl_die(isl_schedule_band_get_ctx(band), isl_error_invalid,
 			"invalid member position", return isl_bool_error);
 
-	return band->coincident[pos];
+	return isl_bool_ok(band->coincident[pos]);
 }
 
 /* Mark the given scheduling dimension as being coincident or not
@@ -272,7 +274,7 @@ isl_bool isl_schedule_band_get_permutable(__isl_keep isl_schedule_band *band)
 {
 	if (!band)
 		return isl_bool_error;
-	return band->permutable;
+	return isl_bool_ok(band->permutable);
 }
 
 /* Mark the schedule band permutable or not according to "permutable"?
@@ -371,7 +373,7 @@ enum isl_ast_loop_type isl_schedule_band_member_get_ast_loop_type(
 
 	if (pos < 0 || pos >= band->n)
 		isl_die(isl_schedule_band_get_ctx(band), isl_error_invalid,
-			"invalid member position", return -1);
+			"invalid member position", return isl_ast_loop_error);
 
 	if (!band->loop_type)
 		return isl_ast_loop_default;
@@ -426,7 +428,7 @@ enum isl_ast_loop_type isl_schedule_band_member_get_isolate_ast_loop_type(
 
 	if (pos < 0 || pos >= band->n)
 		isl_die(isl_schedule_band_get_ctx(band), isl_error_invalid,
-			"invalid member position", return -1);
+			"invalid member position", return isl_ast_loop_error);
 
 	if (!band->isolate_loop_type)
 		return isl_ast_loop_default;
@@ -571,18 +573,30 @@ __isl_give isl_union_set *isl_schedule_band_get_ast_build_options(
 	return options;
 }
 
-/* Does "uset" contain any set that satisfies "is"?
- * "is" is assumed to set its integer argument to 1 if it is satisfied.
+/* Internal data structure for not().
  */
-static int has_any(__isl_keep isl_union_set *uset,
-	isl_stat (*is)(__isl_take isl_set *set, void *user))
+struct isl_not_data {
+	isl_bool (*is)(__isl_keep isl_set *set);
+};
+
+/* Does "set" not satisfy data->is()?
+ */
+static isl_bool not(__isl_keep isl_set *set, void *user)
 {
-	int found = 0;
+	struct isl_not_data *data = user;
 
-	if (isl_union_set_foreach_set(uset, is, &found) < 0 && !found)
-		return -1;
+	return isl_bool_not(data->is(set));
+}
 
-	return found;
+/* Does "uset" contain any set that satisfies "is"?
+ * In other words, is it not the case that all of them do not satisfy "is"?
+ */
+static isl_bool has_any(__isl_keep isl_union_set *uset,
+	isl_bool (*is)(__isl_keep isl_set *set))
+{
+	struct isl_not_data data = { is };
+
+	return isl_bool_not(isl_union_set_every_set(uset, &not, &data));
 }
 
 /* Does "set" live in a space of the form
@@ -590,22 +604,17 @@ static int has_any(__isl_keep isl_union_set *uset,
  *	isolate[[...] -> [...]]
  *
  * ?
- *
- * If so, set *found and abort the search.
  */
-static isl_stat is_isolate(__isl_take isl_set *set, void *user)
+static isl_bool is_isolate(__isl_keep isl_set *set)
 {
-	int *found = user;
-
 	if (isl_set_has_tuple_name(set)) {
 		const char *name;
 		name = isl_set_get_tuple_name(set);
 		if (isl_set_is_wrapping(set) && !strcmp(name, "isolate"))
-			*found = 1;
+			return isl_bool_true;
 	}
-	isl_set_free(set);
 
-	return *found ? isl_stat_error : isl_stat_ok;
+	return isl_bool_false;
 }
 
 /* Does "options" include an option of the ofrm
@@ -614,19 +623,21 @@ static isl_stat is_isolate(__isl_take isl_set *set, void *user)
  *
  * ?
  */
-static int has_isolate_option(__isl_keep isl_union_set *options)
+static isl_bool has_isolate_option(__isl_keep isl_union_set *options)
 {
 	return has_any(options, &is_isolate);
 }
 
 /* Does "set" encode a loop AST generation option?
  */
-static isl_stat is_loop_type_option(__isl_take isl_set *set, void *user)
+static isl_bool is_loop_type_option(__isl_keep isl_set *set)
 {
-	int *found = user;
+	isl_size dim;
 
-	if (isl_set_dim(set, isl_dim_set) == 1 &&
-	    isl_set_has_tuple_name(set)) {
+	dim = isl_set_dim(set, isl_dim_set);
+	if (dim < 0)
+		return isl_bool_error;
+	if (dim == 1 && isl_set_has_tuple_name(set)) {
 		const char *name;
 		enum isl_ast_loop_type type;
 		name = isl_set_get_tuple_name(set);
@@ -634,13 +645,11 @@ static isl_stat is_loop_type_option(__isl_take isl_set *set, void *user)
 		    type <= isl_ast_loop_separate; ++type) {
 			if (strcmp(name, option_str[type]))
 				continue;
-			*found = 1;
-			break;
+			return isl_bool_true;
 		}
 	}
-	isl_set_free(set);
 
-	return *found ? isl_stat_error : isl_stat_ok;
+	return isl_bool_false;
 }
 
 /* Does "set" encode a loop AST generation option for the isolated part?
@@ -650,22 +659,19 @@ static isl_stat is_loop_type_option(__isl_take isl_set *set, void *user)
  *
  * with t equal to "atomic", "unroll" or "separate"?
  */
-static isl_stat is_isolate_loop_type_option(__isl_take isl_set *set, void *user)
+static isl_bool is_isolate_loop_type_option(__isl_keep isl_set *set)
 {
-	int *found = user;
 	const char *name;
 	enum isl_ast_loop_type type;
 	isl_map *map;
 
-	if (!isl_set_is_wrapping(set)) {
-		isl_set_free(set);
-		return isl_stat_ok;
-	}
-	map = isl_set_unwrap(set);
+	if (!isl_set_is_wrapping(set))
+		return isl_bool_false;
+	map = isl_set_unwrap(isl_set_copy(set));
 	if (!isl_map_has_tuple_name(map, isl_dim_in) ||
 	    !isl_map_has_tuple_name(map, isl_dim_out)) {
 		isl_map_free(map);
-		return isl_stat_ok;
+		return isl_bool_false;
 	}
 	name = isl_map_get_tuple_name(map, isl_dim_in);
 	if (!strcmp(name, "isolate")) {
@@ -674,26 +680,26 @@ static isl_stat is_isolate_loop_type_option(__isl_take isl_set *set, void *user)
 		    type <= isl_ast_loop_separate; ++type) {
 			if (strcmp(name, option_str[type]))
 				continue;
-			*found = 1;
-			break;
+			isl_map_free(map);
+			return isl_bool_true;
 		}
 	}
 	isl_map_free(map);
 
-	return *found ? isl_stat_error : isl_stat_ok;
+	return isl_bool_false;
 }
 
 /* Does "options" encode any loop AST generation options
  * for the isolated part?
  */
-static int has_isolate_loop_type_options(__isl_keep isl_union_set *options)
+static isl_bool has_isolate_loop_type_options(__isl_keep isl_union_set *options)
 {
 	return has_any(options, &is_isolate_loop_type_option);
 }
 
 /* Does "options" encode any loop AST generation options?
  */
-static int has_loop_type_options(__isl_keep isl_union_set *options)
+static isl_bool has_loop_type_options(__isl_keep isl_union_set *options)
 {
 	return has_any(options, &is_loop_type_option);
 }
@@ -859,7 +865,7 @@ static __isl_give isl_union_set *clear_isolate_loop_types(
 __isl_give isl_schedule_band *isl_schedule_band_set_ast_build_options(
 	__isl_take isl_schedule_band *band, __isl_take isl_union_set *options)
 {
-	int has_isolate, has_loop_type, has_isolate_loop_type;
+	isl_bool has_isolate, has_loop_type, has_isolate_loop_type;
 
 	band = isl_schedule_band_cow(band);
 	if (!band || !options)
@@ -1076,7 +1082,8 @@ static isl_multi_union_pw_aff *isl_multi_union_pw_aff_tile(
 	__isl_take isl_multi_val *sizes)
 {
 	isl_ctx *ctx;
-	int i, n;
+	int i;
+	isl_size n;
 	isl_val *v;
 	int scale;
 
@@ -1084,6 +1091,8 @@ static isl_multi_union_pw_aff *isl_multi_union_pw_aff_tile(
 	scale = isl_options_get_tile_scale_tile_loops(ctx);
 
 	n = isl_multi_union_pw_aff_dim(sched, isl_dim_set);
+	if (n < 0)
+		sched = isl_multi_union_pw_aff_free(sched);
 	for (i = 0; i < n; ++i) {
 		isl_union_pw_aff *upa;
 
