@@ -13,6 +13,7 @@
  * CS 42112, 75589 Paris Cedex 12, France
  */
 
+#include <isl/id.h>
 #include <isl/val.h>
 #include <isl/space.h>
 #include <isl/map.h>
@@ -24,8 +25,8 @@
 
 #include <isl_list_templ.h>
 
-#undef BASE
-#define BASE schedule_tree
+#undef EL_BASE
+#define EL_BASE schedule_tree
 
 #include <isl_list_templ.c>
 
@@ -441,7 +442,7 @@ error:
 isl_bool isl_schedule_tree_is_subtree_anchored(
 	__isl_keep isl_schedule_tree *tree)
 {
-	return tree ? tree->anchored : isl_bool_error;
+	return tree ? isl_bool_ok(tree->anchored) : isl_bool_error;
 }
 
 /* Does the root node of "tree" depend on its position in the complete
@@ -486,17 +487,15 @@ int isl_schedule_tree_is_anchored(__isl_keep isl_schedule_tree *tree)
 __isl_give isl_schedule_tree *isl_schedule_tree_update_anchored(
 	__isl_take isl_schedule_tree *tree)
 {
-	int i, n;
+	int i;
+	isl_size n;
 	int anchored;
 
-	if (!tree)
-		return NULL;
-
 	anchored = isl_schedule_tree_is_anchored(tree);
-	if (anchored < 0)
+	n = isl_schedule_tree_n_children(tree);
+	if (anchored < 0 || n < 0)
 		return isl_schedule_tree_free(tree);
 
-	n = isl_schedule_tree_list_n_schedule_tree(tree->children);
 	for (i = 0; !anchored && i < n; ++i) {
 		isl_schedule_tree *child;
 
@@ -630,7 +629,8 @@ isl_bool isl_schedule_tree_plain_is_equal(__isl_keep isl_schedule_tree *tree1,
 	__isl_keep isl_schedule_tree *tree2)
 {
 	isl_bool equal;
-	int i, n;
+	int i;
+	isl_size n1, n2;
 
 	if (!tree1 || !tree2)
 		return isl_bool_error;
@@ -668,7 +668,7 @@ isl_bool isl_schedule_tree_plain_is_equal(__isl_keep isl_schedule_tree *tree1,
 		equal = isl_set_is_equal(tree1->guard, tree2->guard);
 		break;
 	case isl_schedule_node_mark:
-		equal = tree1->mark == tree2->mark;
+		equal = isl_bool_ok(tree1->mark == tree2->mark);
 		break;
 	case isl_schedule_node_leaf:
 	case isl_schedule_node_sequence:
@@ -683,10 +683,13 @@ isl_bool isl_schedule_tree_plain_is_equal(__isl_keep isl_schedule_tree *tree1,
 	if (equal < 0 || !equal)
 		return equal;
 
-	n = isl_schedule_tree_n_children(tree1);
-	if (n != isl_schedule_tree_n_children(tree2))
+	n1 = isl_schedule_tree_n_children(tree1);
+	n2 = isl_schedule_tree_n_children(tree2);
+	if (n1 < 0 || n2 < 0)
+		return isl_bool_error;
+	if (n1 != n2)
 		return isl_bool_false;
-	for (i = 0; i < n; ++i) {
+	for (i = 0; i < n1; ++i) {
 		isl_schedule_tree *child1, *child2;
 
 		child1 = isl_schedule_tree_get_child(tree1, i);
@@ -713,12 +716,16 @@ int isl_schedule_tree_has_children(__isl_keep isl_schedule_tree *tree)
 }
 
 /* Return the number of children of "tree", excluding implicit leaves.
+ * The "children" field is NULL if there are
+ * no children (except for the implicit leaves).
  */
-int isl_schedule_tree_n_children(__isl_keep isl_schedule_tree *tree)
+isl_size isl_schedule_tree_n_children(__isl_keep isl_schedule_tree *tree)
 {
 	if (!tree)
-		return -1;
+		return isl_size_error;
 
+	if (!tree->children)
+		return 0;
 	return isl_schedule_tree_list_n_schedule_tree(tree->children);
 }
 
@@ -766,17 +773,17 @@ __isl_give isl_schedule_tree *isl_schedule_tree_reset_children(
 __isl_give isl_schedule_tree *isl_schedule_tree_drop_child(
 	__isl_take isl_schedule_tree *tree, int pos)
 {
-	int n;
+	isl_size n;
 
 	tree = isl_schedule_tree_cow(tree);
-	if (!tree)
-		return NULL;
 
-	if (!isl_schedule_tree_has_children(tree))
+	n = isl_schedule_tree_n_children(tree);
+	if (n < 0)
+		return isl_schedule_tree_free(tree);
+	if (n == 0)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
 			"tree does not have any explicit children",
 			return isl_schedule_tree_free(tree));
-	n = isl_schedule_tree_list_n_schedule_tree(tree->children);
 	if (pos < 0 || pos >= n)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
 			"position out of bounds",
@@ -808,10 +815,15 @@ __isl_give isl_schedule_tree *isl_schedule_tree_replace_child(
 		goto error;
 
 	if (isl_schedule_tree_is_leaf(child)) {
+		isl_size n;
+
 		isl_schedule_tree_free(child);
 		if (!tree->children && pos == 0)
 			return tree;
-		if (isl_schedule_tree_n_children(tree) != 1)
+		n = isl_schedule_tree_n_children(tree);
+		if (n < 0)
+			return isl_schedule_tree_free(tree);
+		if (n != 1)
 			isl_die(isl_schedule_tree_get_ctx(tree),
 				isl_error_internal,
 				"can only replace single child by leaf",
@@ -946,12 +958,13 @@ __isl_give isl_schedule_tree *isl_schedule_tree_insert_filter(
 __isl_give isl_schedule_tree *isl_schedule_tree_children_insert_filter(
 	__isl_take isl_schedule_tree *tree, __isl_take isl_union_set *filter)
 {
-	int i, n;
-
-	if (!tree || !filter)
-		goto error;
+	int i;
+	isl_size n;
 
 	n = isl_schedule_tree_n_children(tree);
+	if (n < 0 || !filter)
+		goto error;
+
 	for (i = 0; i < n; ++i) {
 		isl_schedule_tree *child;
 
@@ -995,14 +1008,14 @@ __isl_give isl_schedule_tree *isl_schedule_tree_insert_mark(
 
 /* Return the number of members in the band tree root.
  */
-unsigned isl_schedule_tree_band_n_member(__isl_keep isl_schedule_tree *tree)
+isl_size isl_schedule_tree_band_n_member(__isl_keep isl_schedule_tree *tree)
 {
 	if (!tree)
-		return 0;
+		return isl_size_error;
 
 	if (tree->type != isl_schedule_node_band)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
-			"not a band node", return 0);
+			"not a band node", return isl_size_error);
 
 	return isl_schedule_band_n_member(tree->band);
 }
@@ -1527,7 +1540,7 @@ __isl_give isl_id *isl_schedule_tree_mark_get_id(
  */
 static isl_stat set_range_dim(__isl_take isl_map *map, void *user)
 {
-	int *dim = user;
+	isl_size *dim = user;
 
 	*dim = isl_map_dim(map, isl_dim_out);
 	isl_map_free(map);
@@ -1541,15 +1554,17 @@ static isl_stat set_range_dim(__isl_take isl_map *map, void *user)
  *
  * We extract the range dimension from the first map in "umap".
  */
-static int range_dim(__isl_keep isl_union_map *umap)
+static isl_size range_dim(__isl_keep isl_union_map *umap)
 {
-	int dim = -1;
+	isl_size dim = isl_size_error;
+	isl_size n;
 
-	if (!umap)
-		return -1;
-	if (isl_union_map_n_map(umap) == 0)
+	n = isl_union_map_n_map(umap);
+	if (n < 0)
+		return isl_size_error;
+	if (n == 0)
 		isl_die(isl_union_map_get_ctx(umap), isl_error_internal,
-			"unexpected empty input", return -1);
+			"unexpected empty input", return isl_size_error);
 
 	isl_union_map_foreach_map(umap, &set_range_dim, &dim);
 
@@ -1594,18 +1609,20 @@ static __isl_give isl_union_map *append_range(__isl_take isl_union_map *umap,
  * from an extension node, but extension nodes are not supported
  * by the caller and it will error out on them.
  */
-static int domain_less(__isl_keep isl_schedule_tree *tree)
+static isl_bool domain_less(__isl_keep isl_schedule_tree *tree)
 {
 	enum isl_schedule_node_type type;
+	isl_size n;
 
 	type = isl_schedule_tree_get_type(tree);
 	switch (type) {
 	case isl_schedule_node_band:
-		return isl_schedule_tree_band_n_member(tree) == 0;
+		n = isl_schedule_tree_band_n_member(tree);
+		return n < 0 ? isl_bool_error : isl_bool_ok(n == 0);
 	case isl_schedule_node_context:
 	case isl_schedule_node_guard:
 	case isl_schedule_node_mark:
-		return 1;
+		return isl_bool_true;
 	case isl_schedule_node_leaf:
 	case isl_schedule_node_error:
 	case isl_schedule_node_domain:
@@ -1614,11 +1631,11 @@ static int domain_less(__isl_keep isl_schedule_tree *tree)
 	case isl_schedule_node_filter:
 	case isl_schedule_node_set:
 	case isl_schedule_node_sequence:
-		return 0;
+		return isl_bool_false;
 	}
 
 	isl_die(isl_schedule_tree_get_ctx(tree), isl_error_internal,
-		"unhandled case", return 0);
+		"unhandled case", return isl_bool_error);
 }
 
 /* Move down to the first descendant of "tree" that contains any schedule
@@ -1627,7 +1644,9 @@ static int domain_less(__isl_keep isl_schedule_tree *tree)
 __isl_give isl_schedule_tree *isl_schedule_tree_first_schedule_descendant(
 	__isl_take isl_schedule_tree *tree, __isl_keep isl_schedule_tree *leaf)
 {
-	while (domain_less(tree)) {
+	isl_bool down;
+
+	while ((down = domain_less(tree)) == isl_bool_true) {
 		if (!isl_schedule_tree_has_children(tree)) {
 			isl_schedule_tree_free(tree);
 			return isl_schedule_tree_copy(leaf);
@@ -1635,6 +1654,9 @@ __isl_give isl_schedule_tree *isl_schedule_tree_first_schedule_descendant(
 		tree = isl_schedule_tree_child(tree, 0);
 	}
 
+	if (down < 0)
+		return isl_schedule_tree_free(tree);
+
 	return tree;
 }
 
@@ -1709,8 +1731,9 @@ static __isl_give isl_space *extract_space_from_filter_child(
 static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 	__isl_keep isl_schedule_tree *tree, __isl_take isl_union_map *outer)
 {
-	int i, n;
-	int dim;
+	int i;
+	isl_size n;
+	isl_size dim;
 	int separate;
 	isl_ctx *ctx;
 	isl_val *v = NULL;
@@ -1718,18 +1741,14 @@ static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 	isl_space *space;
 	isl_union_map *umap;
 
-	if (!tree)
-		return NULL;
-
-	ctx = isl_schedule_tree_get_ctx(tree);
-	if (!tree->children)
-		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_internal,
-			"missing children", return NULL);
-	n = isl_schedule_tree_list_n_schedule_tree(tree->children);
+	n = isl_schedule_tree_n_children(tree);
+	if (n < 0)
+		return isl_union_map_free(outer);
 	if (n == 0)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_internal,
-			"missing children", return NULL);
+			"missing children", return isl_union_map_free(outer));
 
+	ctx = isl_schedule_tree_get_ctx(tree);
 	separate = n > 1 && (tree->type == isl_schedule_node_sequence ||
 			    isl_options_get_schedule_separate_components(ctx));
 
@@ -1744,14 +1763,16 @@ static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 	mv = isl_multi_val_zero(space);
 
 	dim = isl_multi_val_dim(mv, isl_dim_set);
+	if (dim < 0)
+		umap = isl_union_map_free(umap);
 	for (i = 0; i < n; ++i) {
 		isl_multi_val *mv_copy;
 		isl_union_pw_multi_aff *upma;
 		isl_union_map *umap_i;
 		isl_union_set *dom;
 		isl_schedule_tree *child;
-		int dim_i;
-		int empty;
+		isl_size dim_i;
+		isl_bool empty;
 
 		child = isl_schedule_tree_list_get_schedule_tree(
 							tree->children, i);
@@ -1814,6 +1835,7 @@ static __isl_give isl_union_map *subtree_schedule_extend(
 	isl_multi_union_pw_aff *mupa;
 	isl_union_map *umap;
 	isl_union_set *domain;
+	isl_size n;
 
 	if (!tree)
 		return NULL;
@@ -1831,7 +1853,10 @@ static __isl_give isl_union_map *subtree_schedule_extend(
 	case isl_schedule_node_mark:
 		return subtree_schedule_extend_child(tree, outer);
 	case isl_schedule_node_band:
-		if (isl_schedule_tree_band_n_member(tree) == 0)
+		n = isl_schedule_tree_band_n_member(tree);
+		if (n < 0)
+			return isl_union_map_free(outer);
+		if (n == 0)
 			return subtree_schedule_extend_child(tree, outer);
 		mupa = isl_schedule_band_get_partial_schedule(tree->band);
 		umap = isl_union_map_from_multi_union_pw_aff(mupa);
@@ -1877,14 +1902,14 @@ static __isl_give isl_union_set *initial_domain(
 static __isl_give isl_union_set *initial_domain_from_children(
 	__isl_keep isl_schedule_tree *tree)
 {
-	int i, n;
+	int i;
+	isl_size n;
 	isl_space *space;
 	isl_union_set *domain;
 
-	if (!tree->children)
-		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_internal,
-			"missing children", return NULL);
-	n = isl_schedule_tree_list_n_schedule_tree(tree->children);
+	n = isl_schedule_tree_n_children(tree);
+	if (n < 0)
+		return NULL;
 	if (n == 0)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_internal,
 			"missing children", return NULL);
@@ -1916,6 +1941,7 @@ static __isl_give isl_union_set *initial_domain(
 	isl_multi_union_pw_aff *mupa;
 	isl_union_set *domain;
 	isl_union_map *exp;
+	isl_size n;
 
 	if (!tree)
 		return NULL;
@@ -1940,7 +1966,10 @@ static __isl_give isl_union_set *initial_domain(
 			"cannot construct subtree schedule of tree "
 			"with extension nodes", return NULL);
 	case isl_schedule_node_band:
-		if (isl_schedule_tree_band_n_member(tree) == 0)
+		n = isl_schedule_tree_band_n_member(tree);
+		if (n < 0)
+			return NULL;
+		if (n == 0)
 			isl_die(isl_schedule_tree_get_ctx(tree),
 				isl_error_internal,
 				"0D band should be handled by caller",
@@ -2112,7 +2141,7 @@ __isl_give isl_schedule_tree *isl_schedule_tree_sequence_splice(
 	__isl_take isl_schedule_tree *tree, int pos,
 	__isl_take isl_schedule_tree *child)
 {
-	int n;
+	isl_size n;
 	isl_schedule_tree_list *list1, *list2;
 
 	tree = isl_schedule_tree_cow(tree);
@@ -2122,6 +2151,8 @@ __isl_give isl_schedule_tree *isl_schedule_tree_sequence_splice(
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
 			"not a sequence node", goto error);
 	n = isl_schedule_tree_n_children(tree);
+	if (n < 0)
+		goto error;
 	if (pos < 0 || pos >= n)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
 			"position out of bounds", goto error);
@@ -2275,7 +2306,7 @@ static __isl_give isl_set *isolate_final(__isl_keep isl_set *isolate,
 __isl_give isl_schedule_tree *isl_schedule_tree_band_split(
 	__isl_take isl_schedule_tree *tree, int pos, int depth)
 {
-	int n;
+	isl_size n;
 	isl_set *isolate, *tree_isolate, *child_isolate;
 	isl_schedule_tree *child;
 
@@ -2286,6 +2317,8 @@ __isl_give isl_schedule_tree *isl_schedule_tree_band_split(
 			"not a band node", return isl_schedule_tree_free(tree));
 
 	n = isl_schedule_tree_band_n_member(tree);
+	if (n < 0)
+		return isl_schedule_tree_free(tree);
 	if (pos < 0 || pos > n)
 		isl_die(isl_schedule_tree_get_ctx(tree), isl_error_invalid,
 			"position out of bounds",
@@ -2329,11 +2362,12 @@ __isl_give isl_schedule_tree *isl_schedule_tree_append_to_leaves(
 	__isl_take isl_schedule_tree *tree1,
 	__isl_take isl_schedule_tree *tree2)
 {
-	int i, n;
+	int i;
+	isl_size n;
 
-	if (!tree1 || !tree2)
-		goto error;
 	n = isl_schedule_tree_n_children(tree1);
+	if (n < 0 || !tree2)
+		goto error;
 	if (n == 0) {
 		isl_schedule_tree_list *list;
 		list = isl_schedule_tree_list_from_schedule_tree(tree2);
@@ -2617,16 +2651,23 @@ error:
 
 /* Are any members in "band" marked coincident?
  */
-static int any_coincident(__isl_keep isl_schedule_band *band)
+static isl_bool any_coincident(__isl_keep isl_schedule_band *band)
 {
-	int i, n;
+	int i;
+	isl_size n;
 
 	n = isl_schedule_band_n_member(band);
-	for (i = 0; i < n; ++i)
-		if (isl_schedule_band_member_get_coincident(band, i))
-			return 1;
+	if (n < 0)
+		return isl_bool_error;
+	for (i = 0; i < n; ++i) {
+		isl_bool coincident;
 
-	return 0;
+		coincident = isl_schedule_band_member_get_coincident(band, i);
+		if (coincident < 0 || coincident)
+			return coincident;
+	}
+
+	return isl_bool_false;
 }
 
 /* Print the band node "band" to "p".
@@ -2639,7 +2680,8 @@ static __isl_give isl_printer *print_tree_band(__isl_take isl_printer *p,
 	__isl_keep isl_schedule_band *band)
 {
 	isl_union_set *options;
-	int empty;
+	isl_bool empty;
+	isl_bool coincident;
 
 	p = isl_printer_print_str(p, "schedule");
 	p = isl_printer_yaml_next(p);
@@ -2652,8 +2694,12 @@ static __isl_give isl_printer *print_tree_band(__isl_take isl_printer *p,
 		p = isl_printer_yaml_next(p);
 		p = isl_printer_print_int(p, 1);
 	}
-	if (any_coincident(band)) {
-		int i, n;
+	coincident = any_coincident(band);
+	if (coincident < 0)
+		return isl_printer_free(p);
+	if (coincident) {
+		int i;
+		isl_size n;
 		int style;
 
 		p = isl_printer_yaml_next(p);
@@ -2663,6 +2709,8 @@ static __isl_give isl_printer *print_tree_band(__isl_take isl_printer *p,
 		p = isl_printer_set_yaml_style(p, ISL_YAML_STYLE_FLOW);
 		p = isl_printer_yaml_start_sequence(p);
 		n = isl_schedule_band_n_member(band);
+		if (n < 0)
+			return isl_printer_free(p);
 		for (i = 0; i < n; ++i) {
 			p = isl_printer_print_int(p,
 			    isl_schedule_band_member_get_coincident(band, i));
@@ -2703,7 +2751,8 @@ __isl_give isl_printer *isl_printer_print_schedule_tree_mark(
 	__isl_take isl_printer *p, __isl_keep isl_schedule_tree *tree,
 	int n_ancestor, int *child_pos)
 {
-	int i, n;
+	int i;
+	isl_size n;
 	int sequence = 0;
 	int block;
 
@@ -2791,7 +2840,10 @@ __isl_give isl_printer *isl_printer_print_schedule_tree_mark(
 	}
 	p = isl_printer_yaml_next(p);
 
-	if (!tree->children) {
+	n = isl_schedule_tree_n_children(tree);
+	if (n < 0)
+		return isl_printer_free(p);
+	if (n == 0) {
 		if (n_ancestor > 0 && block) {
 			isl_schedule_tree *leaf;
 
@@ -2813,7 +2865,6 @@ __isl_give isl_printer *isl_printer_print_schedule_tree_mark(
 		p = isl_printer_yaml_next(p);
 	}
 
-	n = isl_schedule_tree_list_n_schedule_tree(tree->children);
 	for (i = 0; i < n; ++i) {
 		isl_schedule_tree *t;
 
